package edu.chl.hajo.monopoly.core;

import java.util.ArrayList;
import java.util.List;
import static org.junit.Assert.assertTrue;
import static edu.chl.hajo.monopoly.core.TestBoard.*;
import org.junit.Before;
import org.junit.Test;

/**
  Her we run integration tests (more participation objects and full use cases)
 @author hajo
 */
public class TestModel {

    // This will later come from GUI
    public static final String PLAYER_NAMES = "olle:fia:allan:siv";
    private Board b;
    private List<Player> ps;

    @Before
    public void before() {
        b = new Board(getSpaces(SPACE_NAMES));
        ps = getPlayers(PLAYER_NAMES);
    }

    @Test
    public void testModelCreated() {  // TODO Not a good name!
        Monopoly m = new Monopoly(ps, b, new Dices());
        List<Player> ps2 = m.getPlayers();
        for (Player p : ps2) {
            assertTrue(p.getPosition().equals(b.getStart()));
        }
        assertTrue(m.getActual().equals(ps.get(0)));
    }

    @Test
    public void testMove() {
        final int i = 6;
        Monopoly m = new Monopoly(ps, b, new MockDices(i));
        m.move();
        Player a = m.getActual();
        // No player change
        assertTrue(m.getActual().equals(ps.get(0)));
        Space s = a.getPosition();
        assertTrue(s.equals(b.getSpace(i)));
    }
    
     @Test
    public void testMoveAndPassGo() {
        MockDices md = new MockDices(35);
        Monopoly m = new Monopoly(ps, b, md);
        m.move();  // At Space 35
        md.setValue(10);
        m.move(); // At Space (35+10) % 40
        assertTrue(m.getActual().getBalance()
                == Monopoly.BALANCE + Monopoly.BONUS);
    }

    @Test
    public void testEndTurn() {
        // For now(?) only thing is to switch player
        final int i = 6;
        Monopoly m = new Monopoly(ps, b, new MockDices(i));
        // Switch many 
        for (Player p : m.getPlayers()) {
            m.next();
        }
        // Same as from start
        assertTrue(m.getActual().equals(ps.get(0)));
    }

  
   // ------  Helpers -------------------
    public static List<Player> getPlayers(String names) {
        List<Player> sps = new ArrayList<>();
        int i = 0;
        for (String name : names.split(":")) {
            sps.add(new Player(name, Piece.values()[i], Monopoly.BALANCE));
            i++;
        }
        return sps;
    }

}
