package edu.chl.hajo.monopoly.core;

import edu.chl.hajo.monopoly.event.Event;
import edu.chl.hajo.monopoly.event.EventBus;
import java.util.ArrayList;
import java.util.List;

/**
 Class representing the overall game

 @author hajo

 */
public class Monopoly {

    // Application wide constants // TODO (possibly configurable )
    public static final int BALANCE = 1000;
    public static final int BONUS = 100;

    private final List<Player> players;
    private final Dices dices;
    private final Board board;
    private Player actual;

    // Players list should be random ordered, first player always set as actual!
    public Monopoly(List<Player> players, Board board, Dices dices) {
        this.dices = dices;
        this.board = board;
        this.players = players;
        setActual(players.get(0));
        for (Player p : this.players) {
            p.setPosition(board.getStart());
        }
    }

    // Must call this (again) after GUI up (to trigger events)!
    public void init() {
        setActual(players.get(0));
        for (Player p : this.players) {
            p.setPosition(board.getStart());
        }
    }

    public void next() {
        dices.reset();
        int i = players.indexOf(getActual());
        setActual(players.get((i + 1) % players.size()));
    }

    public void move() {
        Space oldPos = getActual().getPosition();
        dices.roll();   // TODO GUI updated using observer 
        Space newPos = board.getSpace(oldPos, dices.getTotal());
        getActual().setPosition(newPos);

        // This alternat flow need no interaction, i.e. put it here 
        if (board.passedGo(oldPos, newPos)) {
            getActual().income(BONUS);   // TODO GUI updated using observer 
            EventBus.BUS.publish(new Event(Event.Tag.PLAYER_BALANCE, getActual()));
        }
    }

    // This use case need interaction so split in 2 methods
    public boolean checkPayRent() {
        Space s = getActual().getPosition();
        if (s instanceof IOwnable) {
            IOwnable o = (IOwnable) s;
            return o.getOwner() != Player.NONE && o.getOwner() != getActual();
        } else {
            return false;
        }
    }

    public void payRent() {
        Space s = getActual().getPosition();
        if (s instanceof IOwnable) {
            IOwnable o = (IOwnable) s;
            o.getOwner().income(o.getRent());
            getActual().income(-o.getRent());
        }
    }

    // ------- Accessors -------------
    public Player getActual() {
        return actual;
    }

    public List<Player> getPlayers() {
        return players;
    }

    public Board getBoard() {
        return board;
    }

    public Dices getDices() {
        return dices;
    }

    // -------- Event setters ---------------- 
    // This can be done by NetBeans > Refactor > Encapsulate field
    private void setActual(Player actual) {
        this.actual = actual;
        EventBus.BUS.publish(new Event(Event.Tag.MONOPOLY_NEXT, actual));
    }

}
