package edu.chl.hajo.monopoly.view;

import edu.chl.hajo.monopoly.core.Monopoly;
import edu.chl.hajo.monopoly.core.Player;
import edu.chl.hajo.monopoly.core.Space;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.util.List;
import javax.swing.BorderFactory;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.WindowConstants;

/**

 @author hajo
 */
public class MainWindow extends JPanel {

    private static final int width = 600;  // For game area
    private static final int height = 600;

    public MainWindow() {
        initGui();
    }
   
    public void createGamePanel(Monopoly monopoly) {
        JPanel p = new JPanel();
        p.setLayout(new BorderLayout());
        p.add(createDicesPanel(monopoly), BorderLayout.NORTH);
        p.add(createSpacesPanel(monopoly.getBoard().getSpaces()), BorderLayout.CENTER);
        p.add(createPlayersPanel(monopoly.getPlayers()), BorderLayout.SOUTH);
        this.add(p);
        validate();
    }

    // ----------- Privates -----------------------
    private JPanel createMenuPanel() {
        return new MenuPanel(this);
    }

    private JPanel createDicesPanel(Monopoly monopoly) {
        return new DicePanel(monopoly, width, 1 * height / 10);
    }

    private JPanel createSpacesPanel(List<Space> spaces) {
        JPanel p = new JPanel() {
            /*Image i = Resources.getBackground()   TODO
             .getScaledInstance(width, width, Image.SCALE_SMOOTH);
             @Override
             public void paint(Graphics g) {
             super.paint(g);
             Graphics2D g2 = (Graphics2D) g;              
             g2.drawImage(i, 0, 0, null);
             }*/
        };
        p.setBackground(new Color(255, 255, 230));  // Testing transparency

        p.setPreferredSize(new Dimension(width, 6 * height / 10));
        p.setBorder(BorderFactory.createEtchedBorder());
        int side = (int) Math.sqrt(spaces.size());
        p.setLayout(new GridLayout(4, 10));
        for (Space s : spaces) {
            SpacePanel sp = new SpacePanel(s.getName());
            p.add(sp);
        }
        return p;
    }

    private JPanel createPlayersPanel(List<Player> players) {
        JPanel p = new JPanel();
        p.setPreferredSize(new Dimension(width, 2 * height / 10));
        p.setBorder(BorderFactory.createEtchedBorder());
        for (Player player : players) {
            PlayerPanel pp = createPlayerPanel(player, players.size());
            p.add(pp);
        }
        return p;
    }

    private PlayerPanel createPlayerPanel(Player player, int nPLayers) {
        PlayerPanel p = new PlayerPanel(player, (width / nPLayers) - 10,
                (2 * height / 10) - 10);
        return p;
    }

    private void initGui() {
        setPreferredSize(new Dimension(width, height));
        JFrame window = new JFrame();
        window.getRootPane().setDoubleBuffered(true);
        window.setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
        window.setTitle("MONOPOLY");

        this.setLayout(new BorderLayout());
        this.add(createMenuPanel(), BorderLayout.NORTH);

        window.add(this);
        window.pack();
        window.setLocationRelativeTo(null);
        window.setVisible(true);
    }

}
