package xmlrpc;

import java.io.IOException;

import org.apache.xmlrpc.XmlRpcException;
import org.apache.xmlrpc.server.PropertyHandlerMapping;
import org.apache.xmlrpc.server.XmlRpcServer;
import org.apache.xmlrpc.webserver.WebServer;

/**
 * A receiver using XML-RPC (need Apache XML-RPC 3.1.3)
 * @author hajo
 */

public class Receiver {

	private WebServer ws = null;

	/**
	 * 
	 * @param myHandlerNamer, The name of the object to receive the incoming request
	 * 						(any name will do but must be consistently)
	 * @param myPort, The port to listen to
	 * @param myHandlerClass, The class of the receiver. Will be instantiated
	 * 					to an object handling the request.
	 */
	public Receiver(String myHandlerNamer, String myPort,
			Class<?> myHandlerClass) {
		try {
			PropertyHandlerMapping mapping = new PropertyHandlerMapping();
			mapping.addHandler(myHandlerNamer, myHandlerClass);
			ws = new WebServer(Integer.valueOf(myPort));
			XmlRpcServer server = ws.getXmlRpcServer();
			server.setHandlerMapping(mapping);
		} catch (XmlRpcException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public void start() throws IOException {
		ws.start();
	}

	public void stop() {
		ws.shutdown();
	}

	public int getPort() {
		return ws.getPort();
	}
}
