package chl.hajo.gdxdemo;

import chl.hajo.gdxdemo.core.Bucket;
import chl.hajo.gdxdemo.core.Model;
import chl.hajo.gdxdemo.core.RainDrop;


import com.badlogic.gdx.ApplicationAdapter;
import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Input.Keys;
import com.badlogic.gdx.audio.Music;
import com.badlogic.gdx.audio.Sound;
import com.badlogic.gdx.graphics.GL20;
import com.badlogic.gdx.graphics.OrthographicCamera;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.SpriteBatch;
import com.badlogic.gdx.math.Rectangle;
import com.badlogic.gdx.math.Vector3;
import com.badlogic.gdx.utils.TimeUtils;
import static java.lang.System.out;
import java.util.List;


public class DropView extends ApplicationAdapter {

    public static final int WIDTH = 800;
    public static final int HEIGHT = 480;
    
    // The GLORIOUS model!!!
    private final Model model = new Model(WIDTH, HEIGHT);
    // Graphics
    private final Rectangle bucketBoundingBox = new Rectangle();
    private final Rectangle dropBoundingBox = new Rectangle();
    private Texture dropImage;
    private Texture bucketImage;
    private Sound dropSound;
    private Music rainMusic;
    private SpriteBatch batch;
    private OrthographicCamera camera;
    private long lastDropTime;
    
    @Override
    public void create() {
      
        // load the images for the droplet and the bucket, 64x64 pixels each
        dropImage = new Texture(Gdx.files.internal("droplet.png"));
        bucketImage = new Texture(Gdx.files.internal("bucket.png"));

        // load the drop sound effect and the rain background "music"
        dropSound = Gdx.audio.newSound(Gdx.files.internal("splash.wav"));
        rainMusic = Gdx.audio.newMusic(Gdx.files.internal("rain.wav"));

        // start the playback of the background music immediately
        rainMusic.setLooping(true);
        rainMusic.play();

        // create the camera and the SpriteBatch
        camera = new OrthographicCamera();
        camera.setToOrtho(false, WIDTH, HEIGHT);
        batch = new SpriteBatch();
        model.spawnRaindrop();
    }

    @Override
    public void render() {
        // clear the screen with a dark blue color. The
        // arguments to glClearColor are the red, green
        // blue and alpha component in the range [0,1]
        // of the color to be used to clear the screen.
        Gdx.gl.glClearColor(0, 0, 0.2f, 1);
        Gdx.gl.glClear(GL20.GL_COLOR_BUFFER_BIT);

        // tell the camera to update its matrices.
        camera.update();

        // tell the SpriteBatch to render in the
        // coordinate system specified by the camera.
        batch.setProjectionMatrix(camera.combined);

        // begin a new batch and draw the bucket and all drops
        Bucket bucket = model.getBucket();
        batch.begin();
        batch.draw(bucketImage, bucket.getX(), bucket.getY());
        out.println(model.getRainDrops());
        for (RainDrop d : model.getRainDrops()){
            batch.draw(dropImage, d.getX(), d.getY());
        }
        batch.end();

        processInput();
      
        // check if we need to create a new raindrop
        if (TimeUtils.nanoTime() - lastDropTime > 1000000000) {
            model.spawnRaindrop();
            lastDropTime = TimeUtils.nanoTime();
        }

        // Update model 
        List<RainDrop> drops = model.getRainDrops();
        for (RainDrop d : drops) {
            d.fall(200 * Gdx.graphics.getDeltaTime());
        }
        
        // Collision detection
        bucketBoundingBox.set(bucket.getX(), bucket.getY(),
                bucket.getWidth(), bucket.getHeight());
        for(int i = 0 ; i < drops.size() ; i++ ) {
            RainDrop d = drops.get(i);
            dropBoundingBox.set(d.getX(), d.getY(), d.getWidth(), d.getHeight());
            if (dropBoundingBox.overlaps(bucketBoundingBox)) {
                dropSound.play();
                drops.remove(i);
            }
        }
    }
    
    private void processInput(){
        Bucket bucket = model.getBucket();
        // process user input
        if (Gdx.input.isTouched()) {
            Vector3 touchPos = new Vector3();
            touchPos.set(Gdx.input.getX(), Gdx.input.getY(), 0);
            camera.unproject(touchPos);
            bucket.moveTo(touchPos.x - 64 / 2);
        }
        if (Gdx.input.isKeyPressed(Keys.LEFT)) {
            bucket.moveLeft(200 * Gdx.graphics.getDeltaTime());
        }
        if (Gdx.input.isKeyPressed(Keys.RIGHT)) {
            bucket.moveRight(200 * Gdx.graphics.getDeltaTime());
        }
    }

    @Override
    public void dispose() {
        // dispose of all the native resources
        dropImage.dispose();
        bucketImage.dispose();
        dropSound.dispose();
        rainMusic.dispose();
        batch.dispose();
    }
}
